@@ -2,17 +2,42 @@
 # Author:  Travis Oliphant, 2002
 #
 
-from numpy import *
-from _cephes import *
+from numpy import pi, asarray, floor, isscalar, iscomplex, real, imag, sqrt, \
+        where, mgrid, cos, sin, exp, place, seterr, issubdtype, extract, \
+        complexfloating, less, vectorize, inexact, nan, zeros, sometrue
+from _cephes import ellipkm1, mathieu_a, mathieu_b, iv, jv, gamma, psi, zeta, \
+        hankel1, hankel2, yv, kv, gammaln, errprint, ndtri
 import types
 import specfun
 import orthogonal
 
+__all__ = ['agm', 'ai_zeros', 'assoc_laguerre', 'bei_zeros', 'beip_zeros',
+           'ber_zeros', 'bernoulli', 'berp_zeros', 'bessel_diff_formula',
+           'bi_zeros', 'digamma', 'diric', 'ellipk', 'erf_zeros', 'erfcinv',
+           'erfinv', 'errprint', 'euler', 'fresnel_zeros',
+           'fresnelc_zeros', 'fresnels_zeros', 'gamma', 'gammaln', 'h1vp',
+           'h2vp', 'hankel1', 'hankel2', 'hyp0f1', 'iv', 'ivp', 'jn_zeros',
+           'jnjnp_zeros', 'jnp_zeros', 'jnyn_zeros', 'jv', 'jvp', 'kei_zeros',
+           'keip_zeros', 'kelvin_zeros', 'ker_zeros', 'kerp_zeros', 'kv',
+           'kvp', 'lmbda', 'lpmn', 'lpn', 'lqmn', 'lqn', 'mathieu_a',
+           'mathieu_b', 'mathieu_even_coef', 'mathieu_odd_coef', 'ndtri',
+           'obl_cv_seq', 'pbdn_seq', 'pbdv_seq', 'pbvv_seq',
+           'polygamma', 'pro_cv_seq', 'psi', 'riccati_jn', 'riccati_yn',
+           'sinc', 'sph_harm', 'sph_in', 'sph_inkn',
+           'sph_jn', 'sph_jnyn', 'sph_kn', 'sph_yn', 'y0_zeros', 'y1_zeros',
+           'y1p_zeros', 'yn_zeros', 'ynp_zeros', 'yv', 'yvp', 'zeta']
+
+
 def sinc(x):
     """Returns sin(pi*x)/(pi*x) at all points of array x.
     """
-    w = asarray(asarray(x)*pi)
-    return where(x==0, 1.0, sin(w)/w)
+    w = pi * asarray(x)
+    # w might contain 0, and so temporarily turn off warnings
+    # while calculating sin(w)/w.
+    old_settings = seterr(all='ignore')
+    s = sin(w) / w
+    seterr(**old_settings)
+    return where(x==0, 1.0, s)
 
 def diric(x,n):
     """Returns the periodic sinc function also called the dirichlet function:
@@ -51,22 +76,28 @@ def jnjnp_zeros(nt):
     """Compute nt (<=1200) zeros of the bessel functions Jn and Jn'
     and arange them in order of their magnitudes.
 
-    Outputs (all are arrays of length nt):
+    Returns
+    -------
+    zo[l-1] : ndarray
+        Value of the lth zero of of Jn(x) and Jn'(x). Of length `nt`.
+    n[l-1] : ndarray
+        Order of the Jn(x) or Jn'(x) associated with lth zero. Of length `nt`.
+    m[l-1] : ndarray
+        Serial number of the zeros of Jn(x) or Jn'(x) associated
+        with lth zero. Of length `nt`.
+    t[l-1] : ndarray
+        0 if lth zero in zo is zero of Jn(x), 1 if it is a zero of Jn'(x). Of
+        length `nt`.
 
-       zo[l-1] -- Value of the lth zero of of Jn(x) and Jn'(x)
-       n[l-1]  -- Order of the Jn(x) or Jn'(x) associated with lth zero
-       m[l-1]  -- Serial number of the zeros of Jn(x) or Jn'(x) associated
-                    with lth zero.
-       t[l-1]  -- 0 if lth zero in zo is zero of Jn(x), 1 if it is a zero
-                    of Jn'(x)
-
-    See jn_zeros, jnp_zeros to get separated arrays of zeros.
+    See Also
+    --------
+    jn_zeros, jnp_zeros : to get separated arrays of zeros.
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt>1200):
-        raise ValueError, "Number must be integer <= 1200."
+        raise ValueError("Number must be integer <= 1200.")
     nt = int(nt)
     n,m,t,zo = specfun.jdzo(nt)
-    return zo[:nt],n[:nt],m[:nt],t[:nt]
+    return zo[1:nt+1],n[:nt],m[:nt],t[:nt]
 
 def jnyn_zeros(n,nt):
     """Compute nt zeros of the Bessel functions Jn(x), Jn'(x), Yn(x), and
@@ -75,11 +106,11 @@ def jnyn_zeros(n,nt):
     See jn_zeros, jnp_zeros, yn_zeros, ynp_zeros to get separate arrays.
     """
     if not (isscalar(nt) and isscalar(n)):
-        raise ValueError, "Arguments must be scalars."
+        raise ValueError("Arguments must be scalars.")
     if (floor(n)!=n) or (floor(nt)!=nt):
-        raise ValueError, "Arguments must be integers."
+        raise ValueError("Arguments must be integers.")
     if (nt <=0):
-        raise ValueError, "nt > 0"
+        raise ValueError("nt > 0")
     return specfun.jyzo(abs(n),nt)
 
 def jn_zeros(n,nt):
@@ -104,7 +135,7 @@ def y0_zeros(nt,complex=0):
     of Y0'(z0) = -Y1(z0) at each zero.
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt <=0):
-        raise ValueError, "Arguments must be scalar positive integer."
+        raise ValueError("Arguments must be scalar positive integer.")
     kf = 0
     kc = (complex != 1)
     return specfun.cyzo(nt,kf,kc)
@@ -114,7 +145,7 @@ def y1_zeros(nt,complex=0):
     of Y1'(z1) = Y0(z1) at each zero.
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt <=0):
-        raise ValueError, "Arguments must be scalar positive integer."
+        raise ValueError("Arguments must be scalar positive integer.")
     kf = 1
     kc = (complex != 1)
     return specfun.cyzo(nt,kf,kc)
@@ -124,7 +155,7 @@ def y1p_zeros(nt,complex=0):
     of Y1(z1') at each zero.
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt <=0):
-        raise ValueError, "Arguments must be scalar positive integer."
+        raise ValueError("Arguments must be scalar positive integer.")
     kf = 2
     kc = (complex != 1)
     return specfun.cyzo(nt,kf,kc)
@@ -145,7 +176,7 @@ def jvp(v,z,n=1):
     """Return the nth derivative of Jv(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if n == 0:
         return jv(v,z)
     else:
@@ -156,7 +187,7 @@ def yvp(v,z,n=1):
     """Return the nth derivative of Yv(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if n == 0:
         return yv(v,z)
     else:
@@ -167,7 +198,7 @@ def kvp(v,z,n=1):
     """Return the nth derivative of Kv(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if n == 0:
         return kv(v,z)
     else:
@@ -177,7 +208,7 @@ def ivp(v,z,n=1):
     """Return the nth derivative of Iv(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if n == 0:
         return iv(v,z)
     else:
@@ -187,7 +218,7 @@ def h1vp(v,z,n=1):
     """Return the nth derivative of H1v(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if n == 0:
         return hankel1(v,z)
     else:
@@ -198,7 +229,7 @@ def h2vp(v,z,n=1):
     """Return the nth derivative of H2v(z) with respect to z.
     """
     if not isinstance(n,types.IntType) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if n == 0:
         return hankel2(v,z)
     else:
@@ -210,9 +241,9 @@ def sph_jn(n,z):
     all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z):
@@ -226,9 +257,9 @@ def sph_yn(n,z):
     all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z) or less(z,0):
@@ -242,9 +273,9 @@ def sph_jnyn(n,z):
     derivatives for all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z) or less(z,0):
@@ -259,9 +290,9 @@ def sph_in(n,z):
     all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z):
@@ -275,9 +306,9 @@ def sph_kn(n,z):
     all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z) or less(z,0):
@@ -291,9 +322,9 @@ def sph_inkn(n,z):
     derivatives for all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if iscomplex(z) or less(z,0):
         nm,In,Inp,kn,knp = specfun.csphik(n,z)
     else:
@@ -306,9 +337,9 @@ def riccati_jn(n,x):
     derivative for all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(x)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n == 0): n1 = 1
     else: n1 = n
     nm,jn,jnp = specfun.rctj(n1,x)
@@ -319,9 +350,9 @@ def riccati_yn(n,x):
     derivative for all orders up to and including n.
     """
     if not (isscalar(n) and isscalar(x)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n == 0): n1 = 1
     else: n1 = n
     nm,jn,jnp = specfun.rcty(n1,x)
@@ -380,21 +411,21 @@ def erf_zeros(nt):
     """Compute nt complex zeros of the error function erf(z).
     """
     if (floor(nt)!=nt) or (nt<=0) or not isscalar(nt):
-        raise ValueError, "Argument must be positive scalar integer."
+        raise ValueError("Argument must be positive scalar integer.")
     return specfun.cerzo(nt)
 
 def fresnelc_zeros(nt):
     """Compute nt complex zeros of the cosine fresnel integral C(z).
     """
     if (floor(nt)!=nt) or (nt<=0) or not isscalar(nt):
-        raise ValueError, "Argument must be positive scalar integer."
+        raise ValueError("Argument must be positive scalar integer.")
     return specfun.fcszo(1,nt)
 
 def fresnels_zeros(nt):
     """Compute nt complex zeros of the sine fresnel integral S(z).
     """
     if (floor(nt)!=nt) or (nt<=0) or not isscalar(nt):
-        raise ValueError, "Argument must be positive scalar integer."
+        raise ValueError("Argument must be positive scalar integer.")
     return specfun.fcszo(2,nt)
 
 def fresnel_zeros(nt):
@@ -402,7 +433,7 @@ def fresnel_zeros(nt):
     S(z) and C(z).
     """
     if (floor(nt)!=nt) or (nt<=0) or not isscalar(nt):
-        raise ValueError, "Argument must be positive scalar integer."
+        raise ValueError("Argument must be positive scalar integer.")
     return specfun.fcszo(2,nt), specfun.fcszo(1,nt)
 
 def hyp0f1(v,z):
@@ -440,11 +471,11 @@ def mathieu_even_coef(m,q):
     modified mathieu functions.
     """
     if not (isscalar(m) and isscalar(q)):
-        raise ValueError, "m and q must be scalars."
+        raise ValueError("m and q must be scalars.")
     if (q < 0):
-        raise ValueError, "q >=0"
+        raise ValueError("q >=0")
     if (m != floor(m)) or (m<0):
-        raise ValueError, "m must be an integer >=0."
+        raise ValueError("m must be an integer >=0.")
 
     if (q <= 1):
         qm = 7.5+56.1*sqrt(q)-134.7*q+90.7*sqrt(q)*q
@@ -467,11 +498,11 @@ def mathieu_odd_coef(m,q):
     modified mathieu functions.
     """
     if not (isscalar(m) and isscalar(q)):
-        raise ValueError, "m and q must be scalars."
+        raise ValueError("m and q must be scalars.")
     if (q < 0):
-        raise ValueError, "q >=0"
+        raise ValueError("q >=0")
     if (m != floor(m)) or (m<=0):
-        raise ValueError, "m must be an integer > 0"
+        raise ValueError("m must be an integer > 0")
 
     if (q <= 1):
         qm = 7.5+56.1*sqrt(q)-134.7*q+90.7*sqrt(q)*q
@@ -513,14 +544,14 @@ def lpmn(m,n,z):
     Pmn_z : (m+1, n+1) array
        Values for all orders 0..m and degrees 0..n
     Pmn_d_z : (m+1, n+1) array
-       Derivatives for all orders 0..m and degrees 0..n       
+       Derivatives for all orders 0..m and degrees 0..n
     """
     if not isscalar(m) or (abs(m)>n):
-        raise ValueError, "m must be <= n."
+        raise ValueError("m must be <= n.")
     if not isscalar(n) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if not isscalar(z):
-        raise ValueError, "z must be scalar."
+        raise ValueError("z must be scalar.")
     if (m < 0):
         mp = -m
         mf,nf = mgrid[0:mp+1,0:n+1]
@@ -549,11 +580,11 @@ def lqmn(m,n,z):
     z can be complex.
     """
     if not isscalar(m) or (m<0):
-        raise ValueError, "m must be a non-negative integer."
+        raise ValueError("m must be a non-negative integer.")
     if not isscalar(n) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if not isscalar(z):
-        raise ValueError, "z must be scalar."
+        raise ValueError("z must be scalar.")
     m = int(m)
     n = int(n)
 
@@ -572,7 +603,7 @@ def bernoulli(n):
     """Return an array of the Bernoulli numbers B0..Bn
     """
     if not isscalar(n) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     n = int(n)
     if (n < 2): n1 = 2
     else: n1 = n
@@ -582,7 +613,7 @@ def euler(n):
     """Return an array of the Euler numbers E0..En (inclusive)
     """
     if not isscalar(n) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     n = int(n)
     if (n < 2): n1 = 2
     else:  n1 = n
@@ -595,9 +626,9 @@ def lpn(n,z):
     See also special.legendre  for polynomial class.
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z):
@@ -613,9 +644,9 @@ def lqn(n,z):
     Qn(z) and derivatives for all degrees from 0 to n (inclusive).
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (n!= floor(n)) or (n<0):
-        raise ValueError, "n must be a non-negative integer."
+        raise ValueError("n must be a non-negative integer.")
     if (n < 1): n1 = 1
     else: n1 = n
     if iscomplex(z):
@@ -628,32 +659,32 @@ def ai_zeros(nt):
     """Compute the zeros of Airy Functions Ai(x) and Ai'(x), a and a'
     respectively, and the associated values of Ai(a') and Ai'(a).
 
-    Outputs:
-
-      a[l-1]   -- the lth zero of Ai(x)
-      ap[l-1]  -- the lth zero of Ai'(x)
-      ai[l-1]  -- Ai(ap[l-1])
-      aip[l-1] -- Ai'(a[l-1])
+    Returns
+    -------
+    a[l-1]   -- the lth zero of Ai(x)
+    ap[l-1]  -- the lth zero of Ai'(x)
+    ai[l-1]  -- Ai(ap[l-1])
+    aip[l-1] -- Ai'(a[l-1])
     """
     kf = 1
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be a positive integer scalar."
+        raise ValueError("nt must be a positive integer scalar.")
     return specfun.airyzo(nt,kf)
 
 def bi_zeros(nt):
     """Compute the zeros of Airy Functions Bi(x) and Bi'(x), b and b'
     respectively, and the associated values of Ai(b') and Ai'(b).
 
-    Outputs:
-
-      b[l-1]   -- the lth zero of Bi(x)
-      bp[l-1]  -- the lth zero of Bi'(x)
-      bi[l-1]  -- Bi(bp[l-1])
-      bip[l-1] -- Bi'(b[l-1])
+    Returns
+    -------
+    b[l-1]   -- the lth zero of Bi(x)
+    bp[l-1]  -- the lth zero of Bi'(x)
+    bi[l-1]  -- Bi(bp[l-1])
+    bip[l-1] -- Bi'(b[l-1])
     """
     kf = 2
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be a positive integer scalar."
+        raise ValueError("nt must be a positive integer scalar.")
     return specfun.airyzo(nt,kf)
 
 def lmbda(v,x):
@@ -661,9 +692,9 @@ def lmbda(v,x):
     and their derivatives.  Lv0(x)..Lv(x) are computed with v0=v-int(v).
     """
     if not (isscalar(v) and isscalar(x)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (v<0):
-        raise ValueError, "argument must be > 0."
+        raise ValueError("argument must be > 0.")
     n = int(v)
     v0 = v - n
     if (n < 1): n1 = 1
@@ -680,7 +711,7 @@ def pbdv_seq(v,x):
     their derivatives for Dv0(x)..Dv(x) with v0=v-int(v).
     """
     if not (isscalar(v) and isscalar(x)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     n = int(v)
     v0 = v-n
     if (n < 1): n1=1
@@ -694,7 +725,7 @@ def pbvv_seq(v,x):
     their derivatives for Dv0(x)..Dv(x) with v0=v-int(v).
     """
     if not (isscalar(v) and isscalar(x)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     n = int(v)
     v0 = v-n
     if (n <= 1): n1=1
@@ -708,9 +739,9 @@ def pbdn_seq(n,z):
     their derivatives for D0(z)..Dn(z).
     """
     if not (isscalar(n) and isscalar(z)):
-        raise ValueError, "arguments must be scalars."
+        raise ValueError("arguments must be scalars.")
     if (floor(n)!=n):
-        raise ValueError, "n must be an integer."
+        raise ValueError("n must be an integer.")
     if (abs(n) <= 1):
         n1 = 1
     else:
@@ -722,56 +753,56 @@ def ber_zeros(nt):
     """Compute nt zeros of the kelvin function ber x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,1)
 
 def bei_zeros(nt):
     """Compute nt zeros of the kelvin function bei x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,2)
 
 def ker_zeros(nt):
     """Compute nt zeros of the kelvin function ker x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,3)
 
 def kei_zeros(nt):
     """Compute nt zeros of the kelvin function kei x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,4)
 
 def berp_zeros(nt):
     """Compute nt zeros of the kelvin function ber' x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,5)
 
 def beip_zeros(nt):
     """Compute nt zeros of the kelvin function bei' x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,6)
 
 def kerp_zeros(nt):
     """Compute nt zeros of the kelvin function ker' x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,7)
 
 def keip_zeros(nt):
     """Compute nt zeros of the kelvin function kei' x
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,8)
 
 def kelvin_zeros(nt):
@@ -781,7 +812,7 @@ def kelvin_zeros(nt):
     (ber, bei, ker, kei, ber', bei', ker', kei')
     """
     if not isscalar(nt) or (floor(nt)!=nt) or (nt<=0):
-        raise ValueError, "nt must be positive integer scalar."
+        raise ValueError("nt must be positive integer scalar.")
     return specfun.klvnzo(nt,1), \
            specfun.klvnzo(nt,2), \
            specfun.klvnzo(nt,3), \
@@ -797,11 +828,11 @@ def pro_cv_seq(m,n,c):
     parameter c.
     """
     if not (isscalar(m) and isscalar(n) and isscalar(c)):
-        raise ValueError, "Arguments must be scalars."
+        raise ValueError("Arguments must be scalars.")
     if (n!=floor(n)) or (m!=floor(m)):
-        raise ValueError, "Modes must be integers."
+        raise ValueError("Modes must be integers.")
     if (n-m > 199):
-        raise ValueError, "Difference between n and m is too large."
+        raise ValueError("Difference between n and m is too large.")
     maxL = n-m+1
     return specfun.segv(m,n,c,1)[1][:maxL]
 
@@ -811,14 +842,22 @@ def obl_cv_seq(m,n,c):
     parameter c.
     """
     if not (isscalar(m) and isscalar(n) and isscalar(c)):
-        raise ValueError, "Arguments must be scalars."
+        raise ValueError("Arguments must be scalars.")
     if (n!=floor(n)) or (m!=floor(m)):
-        raise ValueError, "Modes must be integers."
+        raise ValueError("Modes must be integers.")
     if (n-m > 199):
-        raise ValueError, "Difference between n and m is too large."
+        raise ValueError("Difference between n and m is too large.")
     maxL = n-m+1
     return specfun.segv(m,n,c,-1)[1][:maxL]
 
+def ellipk(m):
+    """y=ellipk(m) returns the complete integral of the first kind:
+    integral(1/sqrt(1-m*sin(t)**2),t=0..pi/2)
+
+    This function is rather imprecise around m==1. For more precision
+    around this point, use ellipkm1."""
+    return ellipkm1(1 - asarray(m))
+
 def agm(a,b):
     """Arithmetic, Geometric Mean
 
@@ -833,7 +872,5 @@ def agm(a,b):
     agm(a,a) = a
     min(a,b) < agm(a,b) < max(a,b)
     """
-    res1 = a+b+0.0
-    res2 = a-b
-    k = res2 / res1
-    return res1*pi/4/ellipk(k**2)
+    s = a + b + 0.0
+    return (pi / 4) * s / ellipkm1(4 * a * b / s ** 2)
